@@ -3,6 +3,7 @@
 
 #include "index-storage.h"
 #include "imapc-settings.h"
+#include "imapc-client.h"
 
 #define IMAPC_STORAGE_NAME "imapc"
 #define IMAPC_LIST_ESCAPE_CHAR '%'
@@ -51,6 +52,14 @@ struct imapc_storage_client {
 
 	ARRAY(struct imapc_storage_event_callback) untagged_callbacks;
 
+	/* IMAPC_COMMAND_STATE_OK if no auth failure (yet), otherwise result to
+	   the LOGIN/AUTHENTICATE command. */
+	enum imapc_command_state auth_failed_state;
+	char *auth_failed_reason;
+
+	/* Authentication reply was received (success or failure) */
+	bool auth_returned:1;
+	/* Authentication failed */
 	unsigned int auth_failed:1;
 	unsigned int destroying:1;
 };
@@ -87,6 +96,7 @@ struct imapc_mailbox {
 	struct mailbox box;
 	struct imapc_storage *storage;
 	struct imapc_client_mailbox *client_box;
+	enum imapc_capability capabilities;
 
 	struct mail_index_transaction *delayed_sync_trans;
 	struct mail_index_view *sync_view, *delayed_sync_view;
@@ -109,7 +119,7 @@ struct imapc_mailbox {
 	uint32_t highest_nonrecent_uid;
 
 	ARRAY(uint64_t) rseq_modseqs;
-	ARRAY_TYPE(uint32_t) delayed_expunged_uids;
+	ARRAY_TYPE(seq_range) delayed_expunged_uids;
 	uint32_t sync_uid_validity;
 	uint32_t sync_uid_next;
 	uint64_t sync_highestmodseq;
@@ -148,6 +158,7 @@ int imapc_storage_client_create(struct mail_namespace *ns,
 				struct imapc_storage_client **client_r,
 				const char **error_r);
 void imapc_storage_client_unref(struct imapc_storage_client **client);
+bool imapc_storage_client_handle_auth_failure(struct imapc_storage_client *client);
 
 struct mail_save_context *
 imapc_save_alloc(struct mailbox_transaction_context *_t);
@@ -167,7 +178,7 @@ void imapc_mailbox_run_nofetch(struct imapc_mailbox *mbox);
 void imapc_mail_cache_free(struct imapc_mail_cache *cache);
 int imapc_mailbox_select(struct imapc_mailbox *mbox);
 
-bool imapc_storage_has_modseqs(struct imapc_storage *storage);
+bool imapc_mailbox_has_modseqs(struct imapc_mailbox *mbox);
 bool imap_resp_text_code_parse(const char *str, enum mail_error *error_r);
 void imapc_copy_error_from_reply(struct imapc_storage *storage,
 				 enum mail_error default_error,
