@@ -4,6 +4,8 @@
 #include "file-dotlock.h"
 #include "mail-storage-private.h"
 #include "mail-index-private.h"
+#include "mailbox-recent-flags.h" /* FIXME: remove in v2.3 */
+#include "mailbox-watch.h"
 
 #define MAILBOX_FULL_SYNC_INTERVAL 5
 
@@ -16,30 +18,30 @@ enum mailbox_lock_notify_type {
 	MAILBOX_LOCK_NOTIFY_MAILBOX_OVERRIDE
 };
 
-struct index_vsize_header {
-	uint64_t vsize;
-	uint32_t highest_uid;
-	uint32_t message_count;
+enum index_storage_list_change {
+	INDEX_STORAGE_LIST_CHANGE_ERROR = -1,
+	INDEX_STORAGE_LIST_CHANGE_NONE = 0,
+	INDEX_STORAGE_LIST_CHANGE_INMEMORY,
+	INDEX_STORAGE_LIST_CHANGE_NORECORD,
+	INDEX_STORAGE_LIST_CHANGE_NOT_IN_FS,
+	INDEX_STORAGE_LIST_CHANGE_SIZE_CHANGED,
+	INDEX_STORAGE_LIST_CHANGE_MTIME_CHANGED
 };
 
 struct index_mailbox_context {
 	union mailbox_module_context module_ctx;
 	enum mail_index_open_flags index_flags;
 
-	struct timeout *notify_to, *notify_delay_to;
-	struct index_notify_file *notify_files;
-        struct index_notify_io *notify_ios;
-
 	time_t next_lock_notify; /* temporary */
 	enum mailbox_lock_notify_type last_notify_type;
 
 	const ARRAY_TYPE(keywords) *keyword_names;
 	struct mail_cache_field *cache_fields;
 
-	ARRAY_TYPE(seq_range) recent_flags;
-	uint32_t recent_flags_prev_uid, recent_flags_last_check_nextuid;
-	uint32_t recent_flags_count;
-	uint32_t vsize_hdr_ext_id;
+	struct mailbox_vsize_update *vsize_update;
+
+	uint32_t recent_flags_prev_first_recent_uid;
+	uint32_t recent_flags_last_check_nextuid;
 
 	time_t sync_last_check;
 	uint32_t list_index_sync_ext_id;
@@ -62,7 +64,8 @@ void index_storage_mailbox_alloc(struct mailbox *box, const char *vname,
 int index_storage_mailbox_exists(struct mailbox *box, bool auto_boxes,
 				 enum mailbox_existence *existence_r);
 int index_storage_mailbox_exists_full(struct mailbox *box, const char *subdir,
-				      enum mailbox_existence *existence_r);
+				      enum mailbox_existence *existence_r)
+	ATTR_NULL(2);
 int index_storage_mailbox_open(struct mailbox *box, bool move_to_memory);
 int index_storage_mailbox_enable(struct mailbox *box,
 				 enum mailbox_feature feature);
@@ -70,26 +73,27 @@ void index_storage_mailbox_close(struct mailbox *box);
 void index_storage_mailbox_free(struct mailbox *box);
 int index_storage_mailbox_update(struct mailbox *box,
 				 const struct mailbox_update *update);
-void index_storage_mailbox_update_cache(struct mailbox *box,
+int index_storage_mailbox_update_common(struct mailbox *box,
 					const struct mailbox_update *update);
+int index_storage_mailbox_create(struct mailbox *box, bool directory);
+int index_storage_mailbox_delete_pre(struct mailbox *box);
+int index_storage_mailbox_delete_post(struct mailbox *box);
 int index_storage_mailbox_delete(struct mailbox *box);
 int index_storage_mailbox_delete_dir(struct mailbox *box, bool mailbox_deleted);
-int index_storage_mailbox_rename(struct mailbox *src, struct mailbox *dest,
-				 bool rename_children);
+int index_storage_mailbox_rename(struct mailbox *src, struct mailbox *dest);
+
+int index_mailbox_update_last_temp_file_scan(struct mailbox *box);
+int index_mailbox_fix_inconsistent_existence(struct mailbox *box,
+					     const char *path);
 
 bool index_storage_is_readonly(struct mailbox *box);
 bool index_storage_is_inconsistent(struct mailbox *box);
 
-void index_mailbox_set_recent_uid(struct mailbox *box, uint32_t uid);
-void index_mailbox_set_recent_seq(struct mailbox *box,
-				  struct mail_index_view *view,
-				  uint32_t seq1, uint32_t seq2);
-bool index_mailbox_is_recent(struct mailbox *box, uint32_t uid);
-unsigned int index_mailbox_get_recent_count(struct mailbox *box);
-void index_mailbox_reset_uidvalidity(struct mailbox *box);
-
-void index_mailbox_check_add(struct mailbox *box, const char *path);
-void index_mailbox_check_remove_all(struct mailbox *box);
+/* FIXME: for backwards compatibility - remove in v2.3 */
+#define index_mailbox_set_recent_seq(box, view, seq1, seq2) \
+	mailbox_recent_flags_set_seqs(box, view, seq1, seq2)
+#define index_mailbox_check_add(box, path) mailbox_watch_add(box, path)
+#define index_mailbox_check_remove_all(box) mailbox_watch_remove_all(box)
 
 enum mail_index_sync_flags index_storage_get_sync_flags(struct mailbox *box);
 bool index_mailbox_want_full_sync(struct mailbox *box,
@@ -108,9 +112,30 @@ void index_sync_update_recent_count(struct mailbox *box);
 int index_storage_get_status(struct mailbox *box,
 			     enum mailbox_status_items items,
 			     struct mailbox_status *status_r);
+void index_storage_get_open_status(struct mailbox *box,
+				   enum mailbox_status_items items,
+				   struct mailbox_status *status_r);
 int index_mailbox_get_metadata(struct mailbox *box,
 			       enum mailbox_metadata_items items,
 			       struct mailbox_metadata *metadata_r);
+int index_mailbox_get_virtual_size(struct mailbox *box,
+				   struct mailbox_metadata *metadata_r);
+int index_mailbox_get_physical_size(struct mailbox *box,
+				    struct mailbox_metadata *metadata_r);
+
+int index_storage_attribute_set(struct mailbox_transaction_context *t,
+				enum mail_attribute_type type, const char *key,
+				const struct mail_attribute_value *value);
+int index_storage_attribute_get(struct mailbox_transaction_context *t,
+				enum mail_attribute_type type, const char *key,
+				struct mail_attribute_value *value_r);
+struct mailbox_attribute_iter *
+index_storage_attribute_iter_init(struct mailbox *box,
+				  enum mail_attribute_type type,
+				  const char *prefix);
+const char *
+index_storage_attribute_iter_next(struct mailbox_attribute_iter *iter);
+int index_storage_attribute_iter_deinit(struct mailbox_attribute_iter *iter);
 
 struct mail_search_context *
 index_storage_search_init(struct mailbox_transaction_context *t,
@@ -129,6 +154,7 @@ index_transaction_begin(struct mailbox *box,
 void index_transaction_init(struct mailbox_transaction_context *t,
 			    struct mailbox *box,
 			    enum mailbox_transaction_flags flags);
+void index_transaction_init_pvt(struct mailbox_transaction_context *t);
 int index_transaction_commit(struct mailbox_transaction_context *t,
 			     struct mail_transaction_commit_changes *changes_r);
 void index_transaction_rollback(struct mailbox_transaction_context *t);
@@ -136,6 +162,7 @@ void index_save_context_free(struct mail_save_context *ctx);
 void index_copy_cache_fields(struct mail_save_context *ctx,
 			     struct mail *src_mail, uint32_t dest_seq);
 int index_storage_set_subscribed(struct mailbox *box, bool set);
+void index_storage_destroy(struct mail_storage *storage);
 
 bool index_keyword_array_cmp(const ARRAY_TYPE(keyword_indexes) *k1,
 			     const ARRAY_TYPE(keyword_indexes) *k2);
@@ -143,8 +170,21 @@ bool index_keyword_array_cmp(const ARRAY_TYPE(keyword_indexes) *k1,
 int index_storage_list_index_has_changed(struct mailbox *box,
 					 struct mail_index_view *list_view,
 					 uint32_t seq);
+enum index_storage_list_change
+index_storage_list_index_has_changed_full(struct mailbox *box,
+					  struct mail_index_view *list_view,
+					  uint32_t seq);
 void index_storage_list_index_update_sync(struct mailbox *box,
 					  struct mail_index_transaction *trans,
 					  uint32_t seq);
 
+int index_storage_expunged_sync_begin(struct mailbox *box,
+				      struct mail_index_sync_ctx **ctx_r,
+				      struct mail_index_view **view_r,
+				      struct mail_index_transaction **trans_r,
+				      enum mail_index_sync_flags flags);
+void index_storage_expunging_deinit(struct mailbox *box);
+
+void index_storage_save_abort_last(struct mail_save_context *ctx, uint32_t seq);
+
 #endif
